==> sage-v0.3-2005-05-02-src.tar: sage-0.3/sage/rings/rational_field.py <==
"""
The field of rational numbers.
"""

import random
import ring
import sage.rings.rational


_obj = {}
class _uniq(object):
    def __new__(cls):
        if _obj.has_key(0):
            return _obj[0]
        O = object.__new__(cls)
        _obj[0] = O
        return O

class RationalField(_uniq, ring.Ring):
    """
    The field of rational numbers.
    
    EXAMPLES:
    We create the rational numbers and call a few functions:
        >>> Q = RationalField(); Q
        Rational Field
        >>> Q.characteristic()
        0
        >>> Q.is_field()
        True
        >>> Q.zeta()
        -1
        
    We illustrate arithmetic in Q.
        >>> Q('49/7')
        7
        >>> type(Q('49/7'))
        <type 'rational.Rational'>
        >>> a = Q('19/374'); b = frac(17, 371); print a, b
        19/374 17/371
        >>> a + b
        13407/138754
        >>> b + a
        13407/138754
        >>> a * b
        19/8162
        >>> b * a
        19/8162
        >>> a - b
        691/138754
        >>> b - a
        -691/138754
        >>> a / b
        7049/6358
        >>> b / a
        6358/7049
        >>> b < a
        True
        >>> a < b
        False
        
    Next we illustrate arithmetic with automatic coercion.
    The types that coerce are: str, int, long, Integer.
        >>> a + '17/371'
        13407/138754
        >>> a * 374
        19
        >>> 374 * a
        19
        >>> a/19
        1/374
        >>> a + 1
        393/374
    """
    def __repr__(self):
        return "Rational Field"
    
    def __call__(self, x):
        return sage.rings.rational.Rational(x)

    def __contains__(self, x):
        return isinstance(x, sage.rings.rational.Rational)
    
    def is_field(self):
        return True
        
    def is_atomic_repr(self):
        return True
    
    def name(self):
        return "Q"
    
    def characteristic(self):
        return 0
    
    def gen(self):
        return self(1)
        
    def ngens(self):
        return 1
    
    def random(self, num_bound=1, den_bound=1):
        return self("%s/%s"%(random.randrange(-num_bound-1, num_bound+1),random.randrange(1,den_bound+1)))
    
    def __cmp__(self, other):
        if isinstance(other, RationalField):
            return 0
        return -1
    
    def zeta(self):
        return sage.rings.rational.Rational(-1)

Q = RationalField()
def frac(n,d):
    return sage.rings.rational.Rational(n)/sage.rings.rational.Rational(d)

def factor(x):
    assert isinstance(x, sage.rings.rational.Rational)
    return x.numerator().factor() + \
           [(p,-e) for p, e in x.denominator().factor()]

    
if __name__ ==  '__main__':
    import doctest, sys
    doctest.testmod(sys.modules[__name__])

==> sage-v0.3-2005-06-02-src.tar: sage-0.3/sage/rings/rational_field.py <==
"""
The field of rational numbers.
"""

import random
import ring
import sage.rings.rational


_obj = {}
class _uniq(object):
    def __new__(cls):
        if _obj.has_key(0):
            return _obj[0]
        O = object.__new__(cls)
        _obj[0] = O
        return O

class RationalField(_uniq, ring.Ring):
    """
    The field of rational numbers.
    
    EXAMPLES:
    We create the rational numbers and call a few functions:
        >>> Q = RationalField(); Q
        Rational Field
        >>> Q.characteristic()
        0
        >>> Q.is_field()
        True
        >>> Q.zeta()
        -1
        
    We illustrate arithmetic in Q.
        >>> Q('49/7')
        7
        >>> type(Q('49/7'))
        <type '_rational.Rational'>
        >>> a = Q('19/374'); b = frac(17, 371); print a, b
        19/374 17/371
        >>> a + b
        13407/138754
        >>> b + a
        13407/138754
        >>> a * b
        19/8162
        >>> b * a
        19/8162
        >>> a - b
        691/138754
        >>> b - a
        -691/138754
        >>> a / b
        7049/6358
        >>> b / a
        6358/7049
        >>> b < a
        True
        >>> a < b
        False
        
    Next we illustrate arithmetic with automatic coercion.
    The types that coerce are: str, int, long, Integer.
        >>> a + '17/371'
        13407/138754
        >>> a * 374
        19
        >>> 374 * a
        19
        >>> a/19
        1/374
        >>> a + 1
        393/374
    """
    def __repr__(self):
        return "Rational Field"
    
    def __call__(self, x):
        return sage.rings.rational.Rational(x)

    def __contains__(self, x):
        return isinstance(x, sage.rings.rational.Rational)
    
    def is_field(self):
        return True
        
    def is_atomic_repr(self):
        return True
    
    def name(self):
        return "Q"
    
    def characteristic(self):
        return 0
    
    def gen(self):
        return self(1)
        
    def ngens(self):
        return 1
    
    def random(self, num_bound=1, den_bound=1):
        return self("%s/%s"%(random.randrange(-num_bound-1, num_bound+1),random.randrange(1,den_bound+1)))
    
    def __cmp__(self, other):
        if isinstance(other, RationalField):
            return 0
        return -1
    
    def zeta(self):
        return sage.rings.rational.Rational(-1)

Q = RationalField()
def frac(n,d):
    return sage.rings.rational.Rational(n)/sage.rings.rational.Rational(d)

def factor(x):
    assert isinstance(x, sage.rings.rational.Rational)
    return x.numerator().factor() + \
           [(p,-e) for p, e in x.denominator().factor()]

    
if __name__ ==  '__main__':
    import doctest, sys
    doctest.testmod(sys.modules[__name__])
